@@ -11,8 +11,14 @@
 #include <stdlib.h>
 #include <fstream>
 #include <cstring>
+#include <cmath>
+#include <chrono>
+#include <thread>
+
+#if !defined(_WIN32)
 #include <sys/time.h>
 #include <unistd.h>
+#endif
 
 #include "common.h"
 #include "locate.h"
@@ -64,10 +70,6 @@ pthread_t mainthread;
 #include "GLTextures.h"
 #include "EXRFiles.h"
 
-#ifdef HAVE_LIBOPENIMAGEIO
-#include <OpenImageIO/imageio.h>
-#endif
-
 using settings::locateFile;
 using utils::stopWatch;
 
@@ -485,7 +487,11 @@ GLTexture3<float,GL_FLOAT> fromEXR3(
     std::copy(fil3.getData(),fil3.getData()+imSize,std::back_inserter(data));
   }
 
-  return GLTexture3<float,GL_FLOAT> {data.data(),std::tuple<int,int,int>(wi,ht,count),textureNumber,fmt};
+  return GLTexture3<float,GL_FLOAT> {
+          data.data(),
+          std::tuple<int,int,int>(wi,ht,static_cast<int>(count)),textureNumber,
+          fmt
+  };
 }
 
 void initIBL()
@@ -1206,12 +1212,7 @@ void nextframe()
   double seconds=frameTimer.seconds(true);
   delay -= seconds;
   if(delay > 0) {
-    timespec req;
-    timespec rem;
-    req.tv_sec=(unsigned int) delay;
-    req.tv_nsec=(unsigned int) (1.0e9*(delay-req.tv_sec));
-    while(nanosleep(&req,&rem) < 0 && errno == EINTR)
-      req=rem;
+    std::this_thread::sleep_for(std::chrono::duration<double>(delay));
   }
   if(Step) Animate=false;
 }
@@ -1255,10 +1256,12 @@ void display()
     queueExport=false;
   }
   if(!glthread) {
+#if !defined(_WIN32)
     if(Oldpid != 0 && waitpid(Oldpid,NULL,WNOHANG) != Oldpid) {
       kill(Oldpid,SIGHUP);
       Oldpid=0;
     }
+#endif
   }
 }
 
@@ -1323,7 +1326,9 @@ void reshape(int width, int height)
     static bool initialize=true;
     if(initialize) {
       initialize=false;
+#if !defined(_WIN32)
       Signal(SIGUSR1,updateHandler);
+#endif
     }
   }
 
@@ -1924,41 +1929,36 @@ bool NVIDIA()
 #endif /* HAVE_GL */
 
 // angle=0 means orthographic.
-void glrender(const string& prefix, const picture *pic, const string& format,
-              double width, double height, double angle, double zoom,
-              const triple& m, const triple& M, const pair& shift,
-              const pair& margin, double *t, double *tup,
-              double *background, size_t nlightsin, triple *lights,
-              double *diffuse, double *specular, bool view, int oldpid)
+void glrender(GLRenderArgs const& args, int oldpid)
 {
   Iconify=getSetting<bool>("iconify");
 
-  if(zoom == 0.0) zoom=1.0;
+  auto zoomVal = fpclassify(args.zoom) == FP_NORMAL ? args.zoom : 1.0;
 
-  Prefix=prefix;
-  Picture=pic;
-  Format=format;
+  Prefix=args.prefix;
+  Picture=args.pic;
+  Format=args.format;
 
-  nlights0=nlights=nlightsin;
+  nlights0=nlights=args.nlights;
 
-  Lights=lights;
-  Diffuse=diffuse;
-  Specular=specular;
-  View=view;
-  Angle=angle*radians;
-  Zoom0=zoom;
+  Lights=args.lights;
+  Diffuse=args.diffuse;
+  Specular=args.specular;
+  View=args.view;
+  Angle=args.angle*radians;
+  Zoom0=zoomVal;
   Oldpid=oldpid;
-  Shift=shift/zoom;
-  Margin=margin;
+  Shift=args.shift/zoomVal;
+  Margin=args.margin;
   for(size_t i=0; i < 4; ++i)
-    Background[i]=background[i];
+    Background[i]=args.background[i];
 
-  Xmin=m.getx();
-  Xmax=M.getx();
-  Ymin=m.gety();
-  Ymax=M.gety();
-  Zmin=m.getz();
-  Zmax=M.getz();
+  Xmin=args.m.getx();
+  Xmax=args.M.getx();
+  Ymin=args.m.gety();
+  Ymax=args.M.gety();
+  Zmin=args.m.getz();
+  Zmax=args.M.getz();
 
   haveScene=Xmin < Xmax && Ymin < Ymax && Zmin < Zmax;
   orthographic=Angle == 0.0;
@@ -1973,8 +1973,8 @@ void glrender(const string& prefix, const picture *pic, const string& format,
   if(maxTileWidth <= 0) maxTileWidth=1024;
   if(maxTileHeight <= 0) maxTileHeight=768;
 
-  bool v3d=format == "v3d";
-  bool webgl=format == "html";
+  bool v3d=args.format == "v3d";
+  bool webgl=args.format == "html";
   bool format3d=webgl || v3d;
 
 #ifdef HAVE_GL
@@ -2006,10 +2006,10 @@ void glrender(const string& prefix, const picture *pic, const string& format,
 #endif
 
   for(int i=0; i < 16; ++i)
-    T[i]=t[i];
+    T[i]=args.t[i];
 
   for(int i=0; i < 16; ++i)
-    Tup[i]=tup[i];
+    Tup[i]=args.tup[i];
 
   static bool initialized=false;
 
@@ -2026,9 +2026,9 @@ void glrender(const string& prefix, const picture *pic, const string& format,
       if(antialias) expand *= 2.0;
     }
 
-    oWidth=width;
-    oHeight=height;
-    Aspect=width/height;
+    oWidth=args.width;
+    oHeight=args.height;
+    Aspect=args.width/args.height;
 
     // Force a hard viewport limit to work around direct rendering bugs.
     // Alternatively, one can use -glOptions=-indirect (with a performance
@@ -2046,8 +2046,8 @@ void glrender(const string& prefix, const picture *pic, const string& format,
     if(screenHeight <= 0) screenHeight=maxHeight;
     else screenHeight=min(screenHeight,maxHeight);
 
-    fullWidth=(int) ceil(expand*width);
-    fullHeight=(int) ceil(expand*height);
+    fullWidth=(int) ceil(expand*args.width);
+    fullHeight=(int) ceil(expand*args.height);
 
     if(format3d) {
       Width=fullWidth;
@@ -2141,7 +2141,7 @@ void glrender(const string& prefix, const picture *pic, const string& format,
         glutSetOption(GLUT_MULTISAMPLE,multisample);
 #endif
 #endif
-      string title=string(settings::PROGRAM)+": "+prefix;
+      string title=string(PACKAGE_NAME)+": "+args.prefix;
       fpu_trap(false); // Work around FE_INVALID
       window=glutCreateWindow(title.c_str());
       fpu_trap(settings::trap());
@@ -2230,7 +2230,7 @@ void glrender(const string& prefix, const picture *pic, const string& format,
     setBuffers();
   }
 
-  glClearColor(background[0],background[1],background[2],background[3]);
+  glClearColor(args.background[0],args.background[1],args.background[2],args.background[3]);
 
 #ifdef HAVE_LIBGLUT
 #ifndef HAVE_LIBOSMESA
@@ -2289,12 +2289,16 @@ void glrender(const string& prefix, const picture *pic, const string& format,
       if(havewindow) {
         readyAfterExport=true;
 #ifdef HAVE_PTHREAD
+#if !defined(_WIN32)
         pthread_kill(mainthread,SIGUSR1);
+#endif
 #endif
       } else {
         initialized=true;
         readyAfterExport=true;
+#if !defined(_WIN32)
         Signal(SIGUSR1,exportHandler);
+#endif
         exportHandler();
       }
     } else {
