@@ -21,15 +21,7 @@
 #ifdef HAVE_GL
 
 #include <csignal>
-
-#define GLEW_NO_GLU
-
-#ifdef __MSDOS__
-#define GLEW_STATIC
-#define _WIN32
-#endif
-
-#include "GL/glew.h"
+#include <GL/glew.h>
 
 #ifdef __APPLE__
 #define GL_SILENCE_DEPRECATION
@@ -44,23 +36,22 @@
 #endif
 
 #ifdef HAVE_LIBOSMESA
+#ifndef APIENTRY
+#define APIENTRY
+#endif
 #include <GL/osmesa.h>
 #endif
 
 #else
 
-#ifdef __MSDOS__
-#undef _WIN32
-#include <GL/gl.h>
-#include <GL/wglew.h>
-#include <GL/wglext.h>
-#endif
-
 #ifdef HAVE_LIBGLUT
 #include <GL/glut.h>
 #endif
 
 #ifdef HAVE_LIBOSMESA
+#ifndef APIENTRY
+#define APIENTRY
+#endif
 #include <GL/osmesa.h>
 #endif
 
@@ -71,6 +62,11 @@ typedef unsigned int GLuint;
 typedef int GLint;
 typedef float GLfloat;
 typedef double GLdouble;
+typedef unsigned char GLubyte;
+typedef unsigned int GLenum;
+#define GL_POINTS				0x0000
+#define GL_LINES				0x0001
+#define GL_TRIANGLES				0x0004
 #endif
 
 #ifdef HAVE_LIBGLM
@@ -110,21 +106,28 @@ extern bool outlinemode;
 extern bool wireframeMode;
 
 extern bool orthographic;
+
+// 2D bounds
 extern double xmin,xmax;
 extern double ymin,ymax;
-extern double zmin,zmax;
+
+// 3D bounds
+extern double Xmin,Xmax;
+extern double Ymin,Ymax;
+extern double Zmin,Zmax;
+
 extern int fullWidth,fullHeight;
 extern double Zoom0;
 extern double Angle;
 extern camp::pair Shift;
 extern camp::pair Margin;
 
-extern camp::triple *Lights; 
+extern camp::triple *Lights;
 extern size_t nlights;
 extern double *Diffuse;
-extern double *Background;
+extern double Background[4];
 
-struct projection 
+struct projection
 {
 public:
   bool orthographic;
@@ -134,11 +137,11 @@ public:
   double zoom;
   double angle;
   camp::pair viewportshift;
-  
+
   projection(bool orthographic=false, camp::triple camera=0.0,
              camp::triple up=0.0, camp::triple target=0.0,
              double zoom=0.0, double angle=0.0,
-             camp::pair viewportshift=0.0) : 
+             camp::pair viewportshift=0.0) :
     orthographic(orthographic), camera(camera), up(up), target(target),
     zoom(zoom), angle(angle), viewportshift(viewportshift) {}
 };
@@ -150,17 +153,38 @@ GLuint initHDR();
 
 projection camera(bool user=true);
 
-void glrender(const string& prefix, const camp::picture* pic,
-              const string& format, double width, double height, double angle,
-              double zoom, const camp::triple& m, const camp::triple& M,
-              const camp::pair& shift, const camp::pair& margin, double *t,
-              double *background, size_t nlights, camp::triple *lights,
-              double *diffuse, double *specular, bool view, int oldpid=0);
+struct GLRenderArgs
+{
+  string prefix;
+  camp::picture* pic;
+  string format;
+  double width;
+  double height;
+  double angle;
+  double zoom;
+  camp::triple m;
+  camp::triple M;
+  camp::pair shift;
+  camp::pair margin;
+  double *t;
+  double *tup;
+  double *background;
+  size_t nlights;
+  camp::triple *lights;
+  double *diffuse;
+  double *specular;
+  bool view;
+};
+
+void glrender(GLRenderArgs const& args, int oldpid=0);
 
 extern const double *dprojView;
 extern const double *dView;
 
 extern double BBT[9];
+extern double T[16];
+
+extern bool format3dWait;
 
 }
 
@@ -168,18 +192,18 @@ namespace camp {
 
 struct Billboard {
   double cx,cy,cz;
-  
+
   void init(const triple& center) {
     cx=center.getx();
     cy=center.gety();
     cz=center.getz();
   }
-    
+
   triple transform(const triple& v) const {
     double x=v.getx()-cx;
     double y=v.gety()-cy;
     double z=v.getz()-cz;
-    
+
     return triple(x*gl::BBT[0]+y*gl::BBT[3]+z*gl::BBT[6]+cx,
                   x*gl::BBT[1]+y*gl::BBT[4]+z*gl::BBT[7]+cy,
                   x*gl::BBT[2]+y*gl::BBT[5]+z*gl::BBT[8]+cz);
@@ -191,18 +215,15 @@ extern Billboard BB;
 #ifdef HAVE_LIBGLM
 typedef mem::map<CONST Material,size_t> MaterialMap;
 
-extern std::vector<Material> material;
+extern std::vector<Material> materials;
 extern MaterialMap materialMap;
 extern size_t materialIndex;
 extern int MaterialIndex;
-#endif
-
-#ifdef HAVE_GL
 
 extern const size_t Nbuffer; // Initial size of 2D dynamic buffers
 extern const size_t nbuffer; // Initial size of 0D & 1D dynamic buffers
 
-class vertexData 
+class vertexData
 {
 public:
   GLfloat position[3];
@@ -226,7 +247,7 @@ public:
   GLfloat position[3];
   GLfloat normal[3];
   GLint material;
-  GLubyte color[4];
+  GLfloat color[4];
   VertexData() {};
   VertexData(const triple& v, const triple& n) {
     position[0]=v.getx();
@@ -245,10 +266,10 @@ public:
     normal[1]=n.gety();
     normal[2]=n.getz();
     material=MaterialIndex;
-    color[0]=(int)(bytescale*c[0]);
-    color[1]=(int)(bytescale*c[1]);
-    color[2]=(int)(bytescale*c[2]);
-    color[3]=(int)(bytescale*c[3]);
+    color[0]=c[0];
+    color[1]=c[1];
+    color[2]=c[2];
+    color[3]=c[3];
   }
 };
 
@@ -256,7 +277,7 @@ class vertexData0 {
 public:
   GLfloat position[3];
   GLfloat width;
-  GLint  material;
+  GLint material;
   vertexData0() {};
   vertexData0(const triple& v, double width) : width(width) {
     position[0]=v.getx();
@@ -267,7 +288,7 @@ public:
 };
 
 class vertexBuffer {
-public:  
+public:
   GLenum type;
 
   GLuint verticesBuffer;
@@ -295,7 +316,7 @@ public:
                                           materialsBuffer(0),
                                           rendered(false),
                                           partial(false)
-                                          {}
+  {}
 
   void clear() {
     vertices.clear();
@@ -310,22 +331,22 @@ public:
     vertices0.reserve(nbuffer);
   }
 
- void reserve() {
+  void reserve() {
     vertices.reserve(Nbuffer);
     indices.reserve(Nbuffer);
- }
+  }
 
- void Reserve() {
+  void Reserve() {
     Vertices.reserve(Nbuffer);
     indices.reserve(Nbuffer);
- }
-  
+  }
+
 // Store the vertex v and its normal vector n.
   GLuint vertex(const triple &v, const triple& n) {
     size_t nvertices=vertices.size();
     vertices.push_back(vertexData(v,n));
     return nvertices;
-  }     
+  }
 
 // Store the vertex v and its normal vector n, without an explicit color.
   GLuint tvertex(const triple &v, const triple& n) {
@@ -339,14 +360,14 @@ public:
     size_t nvertices=Vertices.size();
     Vertices.push_back(VertexData(v,n,c));
     return nvertices;
-  }     
+  }
 
 // Store the pixel v and its width.
   GLuint vertex0(const triple &v, double width) {
     size_t nvertices=vertices0.size();
     vertices0.push_back(vertexData0(v,width));
     return nvertices;
-  }     
+  }
 
   // append array b onto array a with offset
   void appendOffset(std::vector<GLuint>& a,
@@ -374,12 +395,6 @@ public:
   }
 };
 
-extern GLint pixelShader;
-extern GLint noNormalShader;
-extern GLint materialShader;
-extern GLint colorShader;
-extern GLint transparentShader;
-
 extern vertexBuffer material0Data;   // pixels
 extern vertexBuffer material1Data;   // material Bezier curves
 extern vertexBuffer materialData;    // material Bezier patches & triangles
@@ -387,9 +402,11 @@ extern vertexBuffer colorData;       // colored Bezier patches & triangles
 extern vertexBuffer triangleData;    // opaque indexed triangles
 extern vertexBuffer transparentData; // transparent patches & triangles
 
-void drawBuffer(vertexBuffer& data, GLint shader);
+void drawBuffer(vertexBuffer& data, GLint shader, bool color=false);
 void drawBuffers();
-void clearMaterialBuffer();
+
+void clearMaterials();
+void clearCenters();
 
 typedef void draw_t();
 void setMaterial(vertexBuffer& data, draw_t *draw);
