@@ -17,17 +17,17 @@
 
 #ifndef NOHASH
 
-#ifdef HAVE_UNORDERED_MAP
+#ifdef HAVE_TR1_UNORDERED_MAP
 
 #include <memory>
-#include <unordered_map>
-#define EXT std
+#include <tr1/unordered_map>
+#define EXT std::tr1
 
 #else
 
-#ifdef HAVE_TR1_UNORDERED_MAP
-#include <tr1/unordered_map>
-#define EXT std::tr1
+#ifdef HAVE_UNORDERED_MAP
+#include <unordered_map>
+#define EXT std
 #else
 #define EXT __gnu_cxx
 #include <ext/hash_map>
@@ -42,44 +42,26 @@
 #ifdef __DECCXX_LIBCXX_RH70
 #define CONST
 #else
-#define CONST const  
+#define CONST const
 #endif
-  
+
 #ifdef USEGC
 
 #define GC_THREADS
 #ifdef __clang__
 #define GC_ATTR_EXPLICIT
+#define GC_NOEXCEPT
 #endif
-#include <gc.h>
+#include <gc/gc.h>
 
 #ifdef GC_DEBUG
 extern "C" {
-#include <gc_backptr.h>
+#include <gc/gc_backptr.h>
 }
 #endif
 
-inline void *asy_malloc(size_t n)
-{
-#ifdef GC_DEBUG
-  if(void *mem=GC_debug_malloc_ignore_off_page(n, GC_EXTRAS))
-#else
-    if(void *mem=GC_malloc_ignore_off_page(n))
-#endif
-      return mem;
-  throw std::bad_alloc();
-}
-
-inline void *asy_malloc_atomic(size_t n)
-{
-#ifdef GC_DEBUG
-  if(void *mem=GC_debug_malloc_atomic_ignore_off_page(n, GC_EXTRAS))
-#else
-    if(void *mem=GC_malloc_atomic_ignore_off_page(n))
-#endif
-      return mem;
-  throw std::bad_alloc();
-}
+void* asy_malloc(size_t n);
+void* asy_malloc_atomic(size_t n);
 
 #undef GC_MALLOC
 #undef GC_MALLOC_ATOMIC
@@ -87,8 +69,10 @@ inline void *asy_malloc_atomic(size_t n)
 #define GC_MALLOC(sz) asy_malloc(sz)
 #define GC_MALLOC_ATOMIC(sz) asy_malloc_atomic(sz)
 
-#include <gc_allocator.h>
-#include <gc_cpp.h>
+#include <gc/gc_allocator.h>
+#include <gc/gc_cpp.h>
+
+#define gc_allocator gc_allocator_ignore_off_page
 
 #else // USEGC
 
@@ -105,8 +89,8 @@ inline void* operator new(size_t size, GCPlacement) {
 }
 
 inline void* operator new[](size_t size, GCPlacement) {
-  return operator new(size);
-}
+                           return operator new(size);
+                         }
 
 template<class T>
 struct GC_type_traits {};
@@ -131,7 +115,11 @@ GC_CONTAINER(vector);
 GC_CONTAINER(deque);
 
 template <typename T, typename Container = vector<T> >
-struct stack : public std::stack<T, Container>, public gc {
+struct stack : public std::stack<T, Container>, public gc {};
+
+template <typename T, typename S >
+struct pair : public std::pair<T, S>, public gc {
+  pair(T t, S s) : std::pair<T,S>(t,s) {}
 };
 
 #define PAIR_ALLOC gc_allocator<std::pair<CONST Key,T> > /* space */
@@ -153,15 +141,15 @@ GC_CONTAINER(multimap);
 #undef GC_CONTAINER
 
 #ifndef NOHASH
-#define GC_CONTAINER(KIND)                                              \
-  template <typename Key, typename T,                                   \
-            typename Hash = EXT::hash<Key>,                             \
-            typename Eq = std::equal_to<Key> >                          \
-  struct KIND : public                                                  \
-  EXT::KIND<Key,T,Hash,Eq,PAIR_ALLOC>, public gc {                      \
-    KIND() : EXT::KIND<Key,T,Hash,Eq,PAIR_ALLOC> () {}                  \
-    KIND(size_t n)                                                      \
-      : EXT::KIND<Key,T,Hash,Eq,PAIR_ALLOC> (n) {}                      \
+#define GC_CONTAINER(KIND)                              \
+  template <typename Key, typename T,                   \
+            typename Hash = EXT::hash<Key>,             \
+            typename Eq = std::equal_to<Key> >          \
+  struct KIND : public                                  \
+  EXT::KIND<Key,T,Hash,Eq,PAIR_ALLOC>, public gc {      \
+    KIND() : EXT::KIND<Key,T,Hash,Eq,PAIR_ALLOC> () {}  \
+    KIND(size_t n)                                      \
+      : EXT::KIND<Key,T,Hash,Eq,PAIR_ALLOC> (n) {}      \
   }
 
 GC_CONTAINER(unordered_map);
@@ -184,9 +172,10 @@ typedef std::basic_ostringstream<char,std::char_traits<char>,
                                  gc_allocator<char> > ostringstream;
 typedef std::basic_stringbuf<char,std::char_traits<char>,
                              gc_allocator<char> > stringbuf;
-inline void compact(int x) {GC_set_dont_expand(x);}
+
+void compact(int x);
 #else
-inline void compact(int x) {}
+inline void compact(int) {}
 typedef std::string string;
 typedef std::stringstream stringstream;
 typedef std::istringstream istringstream;
@@ -194,6 +183,8 @@ typedef std::ostringstream ostringstream;
 typedef std::stringbuf stringbuf;
 #endif // USEGC
 
+std::string stdString(string s);
+
 } // namespace mem
 
 #endif
